@@ -1,6 +1,5 @@
-(* $I1: Unison file synchronizer: src/os.ml $ *)
-(* $I2: Last modified by vouillon on Tue, 31 Aug 2004 11:33:38 -0400 $ *)
-(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
+(* Unison file synchronizer: src/os.ml *)
+(* Copyright 1999-2007 (see COPYING for details) *)
 
 (* This file attempts to isolate operating system specific details from the  *)
 (* rest of the program.                                                      *)
@@ -11,8 +10,7 @@ let myCanonicalHostName =
   try Unix.getenv "UNISONLOCALHOSTNAME"
   with Not_found -> Unix.gethostname()
 
-let tempFilePrefix = ".#"
-let backupFileSuffix = ".unison.bak"
+let tempFilePrefix = ".unison."
 let tempFileSuffixFixed = ".unison.tmp"
 let tempFileSuffix = ref tempFileSuffixFixed
 let includeInTempNames s =
@@ -20,46 +18,13 @@ let includeInTempNames s =
     if s = "" then tempFileSuffixFixed
     else "." ^ s ^ tempFileSuffixFixed
 
+let xferDelete = ref (fun (fp,p) -> ())
+let xferRename = ref (fun (fp,p) (ftp,tp) -> ())
 
-(*****************************************************************************)
-(*                             OPTIONS                                       *)
-(*****************************************************************************)
-(* It seems to me that all the options should be put into a single options   *)
-(* file.  I put these here for lack of a better palce.                       *)
-(*****************************************************************************)
-
-let backups =
-  Prefs.createBool "backups" false
-    "keep backup copies of files (see also 'backup')"
-    ("When this flag is {\\tt true}, "
-     ^ "Unison will keep the old version of a file as a backup whenever "
-     ^ "a change is propagated.  These backup files are left in the same "
-     ^ "directory, with extension \\verb|.bak|.  ")
-
-let maxbackups =
-  Prefs.createInt "maxbackups" 2
-    "number of backed up versions of a file"
-    ("This preference specifies the number of backup versions that will "
-     ^ "be kept by unison, for each path that matches the predicate "
-     ^ "\\verb|backup|.  The default is 2.")
-
-let _ = Prefs.alias "maxbackups" "mirrorversions"
-
-let minbackups =
-  Prefs.createInt "minbackups" 1
-    "number of backup version that will be kept regardless of age"
-    ("When a backup exceeds \\verb|maxbackupage| days old, it will be "
-     ^ "deleted during the next sync.  However \\verb|minbackups| versions "
-     ^ "will be kept regardless of age.")
-
-let maxbackupage =
-  Prefs.createInt "maxbackupage" 100
-    "number of days after which backup versions get deleted"
-    ("When a backup exceeds \\verb|maxbackupage| days old, it will be "
-     ^ "deleted during the next sync.  However \\verb|minbackups| versions "
-     ^ "will be kept regardless of age.  The check is made during file "
-     ^ "synchronization.  A value of 0 will keep files regardless of age.")
-
+let initializeXferFunctions del ren =
+  xferDelete := del;
+  xferRename := ren
+      
 
 (*****************************************************************************)
 (*                      QUERYING THE FILESYSTEM                              *)
@@ -75,52 +40,13 @@ let readLink fspath path =
        let abspath = Fspath.concatToString fspath path in
        Unix.readlink abspath)
 
-let isAppleDoubleFile file =
+let rec isAppleDoubleFile file =
   Prefs.read Osx.rsrc &&
   String.length file > 2 && file.[0] = '.' && file.[1] = '_'
 
-(*****************************************************************************)
-(* When we see a backup file, check to see if it should be deleted because   *)
-(* it is too old or because there are too many backup version for a given    *)
-(* file.   <<>>                                                              *)
-
-let backupRe =
-  Str.regexp ( "^.*\\.\\.\\([0-9]+\\)\\." ^ backupFileSuffix ^ "$" )
-
-let rec removeBackupIfUnwanted fspath path = 
-  if Prefs.read backups then begin
-    let absolutePath = Fspath.concatToString fspath path in
-    let stat = Unix.lstat absolutePath in
-    let age =
-      int_of_float ((Unix.time () -. stat.Unix.st_mtime) /. (60.0 *. 60.0)) in
-    let version = 
-      let pathstring = Path.toString path in
-      if Str.string_match backupRe pathstring 0 then
-        int_of_string (Str.matched_group 1 pathstring)
-      else
-        0 in
-    if version > Prefs.read maxbackups && version > Prefs.read minbackups
-    then begin
-      debug (fun ()->
-        Util.msg "File %s ver %d is too high\n" absolutePath version);
-      delete fspath path
-    end else if
-      Prefs.read maxbackupage <> 0 && age > Prefs.read maxbackupage && 
-      version > Prefs.read minbackups
-    then begin
-      debug (fun ()-> Util.msg "File %s age %d is too old\n" absolutePath age);
-      delete fspath path
-    end else begin
-      debug (fun()-> Util.msg "File %s just right\n" absolutePath);
-    end 
-  end
-
 (* Assumes that (fspath, path) is a directory, and returns the list of       *)
-(* children, except for '.' and '..'.  Note that childrenOf and delete are   *)
-(* mutually recursive: this is because one of the side-effects of childrenOf *)
-(* is to delete old files left around by Unison.                             *)
-(* And to get rid of too old or too many backup files.           <<>>        *)
-and childrenOf fspath path =
+(* children, except for '.' and '..'.                                        *)
+let allChildrenOf fspath path =
   Util.convertUnixErrorsToTransient
   "scanning directory"
     (fun () ->
@@ -128,32 +54,66 @@ and childrenOf fspath path =
         let newFile = try Unix.readdir directory with End_of_file -> "" in
         if newFile = "" then children else
         let newChildren =
-          if newFile = "." || newFile = ".." || isAppleDoubleFile newFile then
+          if newFile = "." || newFile = ".." then
             children
-          else if Util.endswith newFile backupFileSuffix then begin
-            let newPath = Path.child path (Name.fromString newFile) in
-            removeBackupIfUnwanted fspath newPath;
-            children
-          end else if
-            Util.endswith newFile tempFileSuffixFixed &&
-            Util.startswith newFile tempFilePrefix
-          then begin
-            if Util.endswith newFile !tempFileSuffix then begin
-              let newPath = Path.child path (Name.fromString newFile) in
-              debug (fun()-> Util.msg "deleting old temp file %s\n"
-                               (Fspath.concatToString fspath newPath));
-              delete fspath newPath
-            end;
-            children
-          end else
+          else
             Name.fromString newFile :: children in
         loop newChildren directory
       in
       let absolutePath = Fspath.concat fspath path in
-      let directory = Fspath.opendir absolutePath in
-      let result = loop [] directory in
-      Unix.closedir directory;
-      result)
+      let directory =
+        try
+          Some (Fspath.opendir absolutePath)
+        with Unix.Unix_error (Unix.ENOENT, _, _) ->
+          (* FIX (in Ocaml): under Windows, when a directory is empty
+             (not even "." and ".."), FindFirstFile fails with
+             ERROR_FILE_NOT_FOUND while ocaml expects the error
+             ERROR_NO_MORE_FILES *)
+          None
+      in
+      match directory with
+        Some directory ->
+          begin try
+            let result = loop [] directory in
+            Unix.closedir directory;
+            result
+          with Unix.Unix_error _ as e ->
+            begin try
+              Unix.closedir directory
+            with Unix.Unix_error _ -> () end;
+            raise e
+          end
+      | None ->
+          [])
+
+(* Assumes that (fspath, path) is a directory, and returns the list of       *)
+(* children, except for temporary files and AppleDouble files.               *)
+let rec childrenOf fspath path =
+  List.filter
+    (fun filename ->
+       let file = Name.toString filename in
+       if isAppleDoubleFile file then
+         false
+(* does it belong to here ? *)
+(*          else if Util.endswith file backupFileSuffix then begin *)
+(*             let newPath = Path.child path filename in *)
+(*             removeBackupIfUnwanted fspath newPath; *)
+(*             false *)
+(*           end  *)
+       else if
+         Util.endswith file tempFileSuffixFixed &&
+         Util.startswith file tempFilePrefix
+       then begin
+         if Util.endswith file !tempFileSuffix then begin
+           let newPath = Path.child path filename in
+           debug (fun()-> Util.msg "deleting old temp file %s\n"
+                            (Fspath.concatToString fspath newPath));
+           delete fspath newPath
+         end;
+         false
+       end else
+         true)
+    (allChildrenOf fspath path)
 
 (*****************************************************************************)
 (*                        ACTIONS ON FILESYSTEM                              *)
@@ -162,50 +122,59 @@ and childrenOf fspath path =
 (* Deletes a file or a directory, but checks before if there is something    *)
 and delete fspath path =
   Util.convertUnixErrorsToTransient
-  "deleting"
+    "deleting"
     (fun () ->
-       let absolutePath = Fspath.concatToString fspath path in
-       match (Fileinfo.get false fspath path).Fileinfo.typ with
-         `DIRECTORY ->
-           Unix.chmod absolutePath 0o700;
-           Safelist.iter
-             (fun child -> delete fspath (Path.child path child))
-             (childrenOf fspath path);
-           Unix.rmdir absolutePath
-       | `FILE ->
-           if Util.osType <> `Unix then
-             Unix.chmod absolutePath 0o600;
-           Unix.unlink absolutePath;
-           if Prefs.read Osx.rsrc then begin
-             let pathDouble = Osx.appleDoubleFile fspath path in
-             if Sys.file_exists pathDouble then
-               Unix.unlink pathDouble
-           end
-       | `SYMLINK ->
+      let absolutePath = Fspath.concatToString fspath path in
+      match (Fileinfo.get false fspath path).Fileinfo.typ with
+        `DIRECTORY ->
+          begin try
+            Unix.chmod absolutePath 0o700
+          with Unix.Unix_error _ -> () end;
+          Safelist.iter
+            (fun child -> delete fspath (Path.child path child))
+            (allChildrenOf fspath path);
+	  (!xferDelete) (fspath, path);
+          Unix.rmdir absolutePath
+      | `FILE ->
+          if Util.osType <> `Unix then begin
+            try
+              Unix.chmod absolutePath 0o600;
+            with Unix.Unix_error _ -> ()
+          end;
+	  (!xferDelete) (fspath, path);
+          Unix.unlink absolutePath;
+          if Prefs.read Osx.rsrc then begin
+            let pathDouble = Osx.appleDoubleFile fspath path in
+            if Sys.file_exists pathDouble then
+              Unix.unlink pathDouble
+          end
+      | `SYMLINK ->
            (* Note that chmod would not do the right thing on links *)
-           Unix.unlink absolutePath
-       | `ABSENT ->
-           ())
-
-let rename sourcefspath sourcepath targetfspath targetpath =
+          Unix.unlink absolutePath
+      | `ABSENT ->
+          ())
+    
+let rename fname sourcefspath sourcepath targetfspath targetpath =
   let source = Fspath.concat sourcefspath sourcepath in
   let source' = Fspath.toString source in
   let target = Fspath.concat targetfspath targetpath in
   let target' = Fspath.toString target in
-  Util.convertUnixErrorsToTransient
-  "renaming"
+  if source' = target' then
+    raise (Util.Transient ("Rename ("^fname^"): identical source and target " ^ source'));
+  Util.convertUnixErrorsToTransient ("renaming " ^ source' ^ " to " ^ target')
     (fun () ->
-       debug (fun() -> Util.msg "rename %s to %s\n" source' target');
-       Unix.rename source' target';
-       if Prefs.read Osx.rsrc then begin
-         let sourceDouble = Osx.appleDoubleFile sourcefspath sourcepath in
-         let targetDouble = Osx.appleDoubleFile targetfspath targetpath in
-         if Sys.file_exists sourceDouble then
-           Unix.rename sourceDouble targetDouble
-         else if Sys.file_exists targetDouble then
-           Unix.unlink targetDouble
-       end)
-
+      debug (fun() -> Util.msg "rename %s to %s\n" source' target');
+      (!xferRename) (sourcefspath, sourcepath) (targetfspath, targetpath);
+      Unix.rename source' target';
+      if Prefs.read Osx.rsrc then begin
+        let sourceDouble = Osx.appleDoubleFile sourcefspath sourcepath in
+        let targetDouble = Osx.appleDoubleFile targetfspath targetpath in
+        if Sys.file_exists sourceDouble then
+          Unix.rename sourceDouble targetDouble
+        else if Sys.file_exists targetDouble then
+          Unix.unlink targetDouble
+      end)
+    
 let symlink =
   if Util.isCygwin || (Util.osType != `Win32) then
     fun fspath path l ->
@@ -223,11 +192,8 @@ let createDir fspath path props =
   Util.convertUnixErrorsToTransient
   "creating directory"
     (fun () ->
-       let rec createDirAndParents d =
-         if not (Sys.file_exists (Filename.dirname d)) then
-           createDirAndParents (Filename.dirname d);
-         Unix.mkdir d (Props.perms props) in
-       createDirAndParents (Fspath.concatToString fspath path))
+       let absolutePath = Fspath.concatToString fspath path in
+       Unix.mkdir absolutePath (Props.perms props))
 
 (*****************************************************************************)
 (*                              FINGERPRINTS                                 *)
@@ -271,7 +237,7 @@ let safeFingerprint fspath path info optDig =
   in
   retryLoop 10 info (* Maximum retries: 10 times *)
     (match optDig with None -> None | Some (d, _) -> Some d)
-    (match optDig with None -> None | Some (_, d) -> Some d)
+    None
 
 let fullfingerprint_to_string (fp,rfp) =
   Printf.sprintf "(%s,%s)" (Fingerprint.toString fp) (Fingerprint.toString rfp)
@@ -336,38 +302,6 @@ let freshPath fspath path prefix suffix =
 let tempPath fspath path =
   freshPath fspath path tempFilePrefix !tempFileSuffix
 
-(* Generates a file name for a backup file.  If backup file already exists,  *)
-(* the old file will be renamed with the count incremented.  The newest      *)
-(* backup file is always path..001..unison.bak and larger numbers mean older *)
-(* files.                                                                    *)
-let backupPath fspath path =
-  let rec f i =
-    let s = Printf.sprintf "..%03d.%s" i backupFileSuffix in
-    let tempPath = Path.addSuffixToFinalName path s in
-    if exists fspath tempPath then rename fspath tempPath fspath (f (i + 1));
-    tempPath
-  in
-  f 1
-
-
-(*****************************************************************************)
-(*                        PARENT VERIFICATION                                *)
-(*****************************************************************************)
-
-let isdir p =
-  try
-    (Fspath.stat p).Unix.LargeFile.st_kind = Unix.S_DIR
-  with Unix.Unix_error _ -> false
-
-let checkThatParentPathIsADir fspath path =
-  if not (exists fspath path) then
-    let (workingDir,realPath) = Fspath.findWorkingDir fspath path in
-    if not (isdir workingDir) then
-      raise (Util.Fatal (Printf.sprintf
-                           "Path %s is not valid because %s is not a directory"
-                           (Fspath.concatToString fspath path)
-                           (Fspath.toString workingDir)))
-
 (*****************************************************************************)
 (*                     INTERRUPTED SYSTEM CALLS                              *)
 (*****************************************************************************)
